import sys, os
from cct._err import err
from graph.boccagraph import *

class BuilderInterface:
    '''A pure interface to different build system implementations.
       Implementors may save some effort by deriving from BuilderDefault instead.
       The general idea is that the Bocca graph tracks SIDL symbols, but the
       builder plugin must be:
       -) informed when new symbols appear.
       -) informed when a symbol or its dependencies SIDL is changed in a way that 
          may cause a generated impl to be out of date.
       And the plugin *may* be capable of responding to:
       -) deletion of a symbol.
       -) renaming of a symbol.
    '''
    
    def __init__(self, modulePath, project):
        '''Constructor
        @param modulePath the directory containing the implementation of the BuilderInterface
        and associated LocationManager.
        '''
        raise NotImplementedError   
    
    def specializeTemplate(self):
        '''At project creation, specialize the general project template to 
        match the current project.
        '''
        raise NotImplementedError   
    
    def rename(self, oldProjectName, newProjectName, oldProjectVersion, newProjectVersion, oldProjectDir):
        '''Update project build system when the project is renamed.'''
        raise NotImplementedError   

    def update(self, buildFilesOnly=False, quiet=False):
        '''Given a new project graph, update build information. 
           Generally this means regenerating some or all of the build scripts in the project.
        '''
        raise NotImplementedError        
    
    def genSIDL(self, vertex):
        '''Generates SIDL files corresponding to the vertex argument.
           Return is status (Unix-like)
        '''
        raise NotImplementedError        

    def genImpls(self, vertex):
        '''Generates Impls corresponding to the vertex argument.
        Returns status code (Unix-like)
        '''
        raise NotImplementedError
    
    def changed(self, vertexList):
        '''This is how the builder is notified of changes to a list of vertices.
        The build system will perform whatever actions are necessary on these
        vertices during a subsequent update call.
        '''
        raise NotImplementedError
    
    def remove(self, vertexList):
        ''' Removes build-relevant information associated with vertices in vertexList.
        Returns 0 upon success, 1 otherwise.
        '''
        raise NotImplementedError
    
        
#################################################################################################
class BuilderDefault(BuilderInterface):
    '''A default implementation of base interface. Some common functionality exists,
       if the LocationManager abstraction is used properly.
       Adds project, locationManager and changedList members to self.
    '''
    
    def __init__(self, modulePath, project):
        '''Constructor
        Must be called from every derived plugin before rest of plugin's init.
        @param modulePath the directory containing the implementation of the BuilderInterface
        '''
        if str(project.__class__) != 'project.Project':
            err("Builder plugin: invoking update on a vertex that is not a Project: " + str(project.__class__))

        self.project = project      
        # Load the local LocationManagerInterface implementation
        try:
            (file,filename,description) = imp.find_module('locations',[modulePath])
            locations = imp.load_module('locations', file, filename, description)
        except ImportError,e:
            err('Builder: Could not import locations module from ' + modulePath + ': ' + str(e))
        try:
            LocationManagerClass = getattr(locations,'LocationManager')
        except AttributeError,e:
            err('Builder: Could not find LocationManager class for current build template in ' + modulePath+ ': ' + str(e))
            
        self.locationManager =self.project.getLocationManager()
        self.changedList = []

        pass
    
    def specializeTemplate(self):
        '''At project creation, specialize the general project template to 
        match the current project.
        @return silently if called on an incomplete derived class.
        '''
        if self.__class__ is BuilderDefault: raise NotImplementedError   
        pass
    
    def rename(self, oldProjectName, newProjectName, oldProjectVersion, newProjectVersion, oldProjectDir):
        '''Update project build system when the project is renamed.
        @return silently if called on an incomplete derived class.
        '''
        if self.__class__ is BuilderDefault: raise NotImplementedError   
        pass

    def update(self, buildFilesOnly=False, quiet=False):
        '''Given a new project graph, update build information. 
        @param buildFilesOnly: Only update files contaning component and port info.
        @return silently if called on an incomplete derived class.
        '''
        if self.__class__ is BuilderDefault: raise NotImplementedError        
        status = 0
        return status
    
    def genSIDL(self, vertex):
        '''Generates SIDL files corresponding to the vertex argument.
        Derived builders may wrap this or reuse as is.
        @return status of createSIDL operation.
        '''
        status = 0
        if vertex.kind in ['class', 'component', 'interface', 'port', 'enum']:
            status = vertex.createSIDL()
        return status

    def genImpls(self, vertex):
        '''Generates Impls corresponding to the vertex argument.
        @return silently if called on an incomplete derived class.
        '''
        if self.__class__ is BuilderDefault: raise NotImplementedError
        status = 0
        return status
    
    def changed(self, vertexList):
        '''This is how the builder is notified of changes to a list of vertices.
        The build system will perform whatever actions are necessary on these
        vertices during a subsequent update call.
        '''
        if self.__class__ is BuilderDefault: raise NotImplementedError
        return self.changedList
    
    def remove(self, vertexList):
        ''' Removes build-relevant information associated with vertices in vertexList.
        Returns 0 upon success, 1 otherwise. Remove is hard; the default result is failed.
        '''
        if self.__class__ is BuilderDefault: raise NotImplementedError
        return 1
    
        
